################################################################
# Procedures 
################################################################
proc remove_string {string_src string_rem} {
    set idx [string first $string_rem $string_src]
	if {$idx == -1} {
		return $string_src
	}
	set idx1 [expr [string length $string_rem] + $idx]
	return [string replace $string_src $idx $idx1]
}

proc replace_string {string_src string_old string_new} {
    set idx [string first $string_old $string_src]
	if {$idx == -1} {
		return $string_src
	}
	set idx1 [expr [string length $string_old] + $idx - 1]
	return [string replace $string_src $idx $idx1 $string_new]
}

################################################################
# Main script 
################################################################

set argv [list C:/Xilinx/SDSoC/2015.2/platforms]

source ../../scripts/adi_env.tcl

################################################################
# Create the modified adi_env.tcl file 
################################################################
file copy -force ../../scripts/adi_env.tcl adi_env1.tcl
set fp [open adi_env1.tcl r]
set file_data [read $fp]
close $fp

set ad_hdl_dir [file normalize $ad_hdl_dir]
regsub -all {../../..} $file_data $ad_hdl_dir file_data

set fp [open adi_env1.tcl w]
puts $fp $file_data
close $fp

################################################################
# Create the modified system_project.tcl file 
################################################################
set fp [open system_project.tcl r]
set file_data [read $fp]
close $fp

regsub -all {../../scripts/} $file_data "" file_data
regsub -all {\$ad_hdl_dir/projects/scripts/adi_board.tcl} $file_data {adi_board.tcl} file_data

set fp [open system_project1.tcl w]
puts $fp $file_data
close $fp

################################################################
# Create the modified system_bd.tcl file
################################################################
file copy -force system_bd.tcl system_bd1.tcl
set fps [open system_bd.tcl r]
set fpd [open system_bd1.tcl w]

while { [gets $fps line] >= 0 } {
    if {[string length $line] > 0} {
		set new_line [file tail $line]
		puts $fpd [concat source $new_line]
	}
}
puts $fpd "source cf_sd_bd.tcl" 

close $fps
close $fpd

################################################################
# Get the rest of the files
################################################################
set cdir [pwd]
puts $cdir
set carrier [file tail $cdir]
puts $carrier
regsub -all /$carrier $cdir "" tmp
set platform [file tail $tmp]
puts $platform

file copy -force $ad_hdl_dir/projects/$platform/common/${platform}_bd.tcl ${platform}_bd.tcl
file copy -force $ad_hdl_dir/projects/common/$carrier/${carrier}_system_bd.tcl ${carrier}_system_bd.tcl 

################################################################
# Update the adi_board.tcl
################################################################
file copy -force ../../scripts/adi_board.tcl adi_board1.tcl
set fps [open ../../scripts/adi_board.tcl r]
set fpd [open adi_board1.tcl w]

set file_data [read $fps]

regsub -all {sys_cpu_clk} $file_data {clkid2} file_data
regsub -all {sys_concat_intc} $file_data {xlconcat} file_data

puts $fpd $file_data 

close $fps
close $fpd

################################################################
# Update the platform_bd.tcl
################################################################
file copy -force ${platform}_bd.tcl ${platform}_bd1.tcl
set fps [open ${platform}_bd.tcl r]
set fpd [open ${platform}_bd1.tcl w]

set file_data [read $fps]

regsub -all {sys_cpu_clk} $file_data {clkid2} file_data
regsub -all {sys_200m_clk} $file_data {clkid3} file_data
regsub -all {xilinx.com:ip:ila:5.0} $file_data {xilinx.com:ip:ila:5.1} file_data
set file_data [remove_string $file_data {ad_connect  util_dac_unpack/dma_data axi_ad9361_dac_dma/fifo_rd_dout}]
set file_data [remove_string $file_data {ad_connect  util_dac_unpack/fifo_valid axi_ad9361_dac_dma/fifo_rd_valid}]

puts $fpd $file_data 

close $fps
close $fpd

################################################################
# Update the carrier_system_bd.tcl
################################################################
file copy -force ${carrier}_system_bd.tcl ${carrier}_system_bd1.tcl
set fps [open ${carrier}_system_bd.tcl r]
set fpd [open ${carrier}_system_bd1.tcl w]

set file_data [read $fps]

regsub -all {sys_concat_intc} $file_data xlconcat file_data
regsub -all {:sys_concat_intc:} $file_data xlconcat file_data
regsub -all {sys_rstgen} $file_data proc_sys_reset_2 file_data
regsub -all {sys_cpu_clk} $file_data {clkid2} file_data
regsub -all {sys_200m_clk} $file_data {clkid3} file_data
regsub -all {sys_100m_resetn} $file_data {proc_sys_reset_2_peripheral_aresetn} file_data
regsub -all {sys_200m_resetn} $file_data {proc_sys_reset_2_interconnect_aresetn} file_data

set file_data [replace_string $file_data {set_property -dict [list CONFIG.PCW_EN_CLK1_PORT {1}] $sys_ps7} \
			  [concat {set_property -dict [list CONFIG.PCW_EN_CLK0_PORT {1}] $sys_ps7} __ \
					  {set_property -dict [list CONFIG.PCW_EN_CLK1_PORT {1}] $sys_ps7} __ \
					  {set_property -dict [list CONFIG.PCW_EN_CLK2_PORT {1}] $sys_ps7} __ \
					  {set_property -dict [list CONFIG.PCW_EN_CLK3_PORT {1}] $sys_ps7}]]

set file_data [replace_string $file_data {CONFIG.PCW_FPGA0_PERIPHERAL_FREQMHZ {100.0}} {CONFIG.PCW_FPGA0_PERIPHERAL_FREQMHZ {166.0}}]
set file_data [replace_string $file_data {CONFIG.PCW_FPGA1_PERIPHERAL_FREQMHZ {200.0}} {CONFIG.PCW_FPGA1_PERIPHERAL_FREQMHZ {142.0}}]
set file_data [replace_string $file_data {set_property -dict [list CONFIG.PCW_FPGA1_PERIPHERAL_FREQMHZ {142.0}] $sys_ps7} \
			  [concat {set_property -dict [list CONFIG.PCW_FPGA1_PERIPHERAL_FREQMHZ {142.0}] $sys_ps7} __ \
					  {set_property -dict [list CONFIG.PCW_FPGA2_PERIPHERAL_FREQMHZ {100.0}] $sys_ps7} __ \
					  {set_property -dict [list CONFIG.PCW_FPGA3_PERIPHERAL_FREQMHZ {200.0}] $sys_ps7}]]
regsub -all { __ } $file_data \n file_data
					  
set file_data [replace_string $file_data {ad_connect  clkid2 sys_ps7/FCLK_CLK0} {ad_connect  clkid2 sys_ps7/FCLK_CLK2}]
set file_data [replace_string $file_data {ad_connect  clkid3 sys_ps7/FCLK_CLK1} {ad_connect  clkid3 sys_ps7/FCLK_CLK3}]
set file_data [replace_string $file_data {intc_index 0} {intc_index 10}]

puts $fpd $file_data

close $fps
close $fpd

################################################################
# Update the system_top.v
################################################################
file copy -force system_top.v system_top1.v
set fps [open system_top.v r]
set fpd [open system_top1.v w]

set file_data [read $fps]

set file_data [remove_string $file_data {    .ps_intr_0 (ps_intrs[0]),}]
set file_data [remove_string $file_data {    .ps_intr_1 (ps_intrs[1]),}]
set file_data [remove_string $file_data {    .ps_intr_2 (ps_intrs[2]),}]
set file_data [remove_string $file_data {    .ps_intr_3 (ps_intrs[3]),}]
set file_data [remove_string $file_data {    .ps_intr_4 (ps_intrs[4]),}]
set file_data [remove_string $file_data {    .ps_intr_5 (ps_intrs[5]),}]
set file_data [remove_string $file_data {    .ps_intr_6 (ps_intrs[6]),}]
set file_data [remove_string $file_data {    .ps_intr_7 (ps_intrs[7]),}]
set file_data [remove_string $file_data {    .ps_intr_8 (ps_intrs[8]),}]
set file_data [remove_string $file_data {    .ps_intr_9 (ps_intrs[9]),}]

puts $fpd $file_data

close $fps
close $fpd

################################################################
# Update the hw.pfm
################################################################
file copy -force ./sdsoc_platform/hw.pfm ./sdsoc_platform/${platform}_${carrier}_hw.pfm
set fps [open ./sdsoc_platform/hw.pfm r]
set fpd [open ./sdsoc_platform/${platform}_${carrier}_hw.pfm w]

set file_data [read $fps]

regsub -all {XXXX} $file_data ${platform}_${carrier} file_data

puts $fpd $file_data

close $fps
close $fpd

################################################################
# Update the sw.pfm
################################################################
file copy -force ./sdsoc_platform/sw.pfm ./sdsoc_platform/${platform}_${carrier}_sw.pfm
set fps [open ./sdsoc_platform/sw.pfm r]
set fpd [open ./sdsoc_platform/${platform}_${carrier}_sw.pfm w]

set file_data [read $fps]

regsub -all {XXXX} $file_data ${platform}_${carrier} file_data

puts $fpd $file_data

close $fps
close $fpd

################################################################
# Copy all the files to the platform directory
################################################################
set platform_dir $argv/${platform}_${carrier}
set vivado_dir $platform_dir/vivado

file delete -force $platform_dir
file mkdir $platform_dir
file mkdir $vivado_dir

file copy -force adi_env1.tcl $vivado_dir/adi_env.tcl
file copy -force adi_board1.tcl $vivado_dir/adi_board.tcl
file copy -force system_project1.tcl $vivado_dir/system_project.tcl
file copy -force system_bd1.tcl $vivado_dir/system_bd.tcl
file copy -force ${platform}_bd1.tcl $vivado_dir/${platform}_bd.tcl
file copy -force ${carrier}_system_bd1.tcl $vivado_dir/${carrier}_system_bd.tcl
file copy -force system_constr.xdc $vivado_dir/system_constr.xdc
file copy -force system_top1.v $vivado_dir/system_top.v

file delete -force adi_env.tcl
file delete -force adi_env1.tcl
file delete -force system_project1.tcl
file delete -force system_bd1.tcl
file delete -force ${platform}_bd.tcl
file delete -force ${platform}_bd1.tcl
file delete -force ${carrier}_system_bd.tcl
file delete -force ${carrier}_system_bd1.tcl
file delete -force system_top1.v

file copy -force ./sdsoc_platform/cf_sd_bd.tcl $vivado_dir/cf_sd_bd.tcl
file copy -force ./sdsoc_platform/${platform}_${carrier}_hw.pfm $platform_dir/${platform}_${carrier}_hw.pfm
file copy -force ./sdsoc_platform/${platform}_${carrier}_sw.pfm $platform_dir/${platform}_${carrier}_sw.pfm

file delete -force ./sdsoc_platform/${platform}_${carrier}_hw.pfm 
file delete -force ./sdsoc_platform/${platform}_${carrier}_sw.pfm

file copy -force ./sdsoc_platform/arm-xilinx-eabi $platform_dir/arm-xilinx-eabi
file copy -force ./sdsoc_platform/arm-xilinx-linux-gnueabi $platform_dir/arm-xilinx-linux-gnueabi
file copy -force ./sdsoc_platform/boot $platform_dir/boot
#file copy -force ./sdsoc_platform/hardware $platform_dir/hardware

file copy -force ../../common/xilinx/sys_wfifo.tcl $vivado_dir/sys_wfifo.tcl

################################################################
# Generate the platform
################################################################
cd $vivado_dir
source ./system_project.tcl